/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action;

import com.google.gson.Gson;
import org.opengroup.osdu.register.model.Action;
import org.opengroup.osdu.register.util.Config;
import org.opengroup.osdu.register.util.RestDescriptor;
import org.opengroup.osdu.register.util.RetrieveApiTestTemplate;
import com.sun.jersey.api.client.ClientResponse;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public abstract class GetActionByIdApiTest extends RetrieveApiTestTemplate {

    public GetActionByIdApiTest() {
        super(new GetActionDescriptor(), new DeleteActionDescriptor(), new CreateActionDescriptor());
    }


    @Override
    protected boolean isDataInTenant() {
        return true;
    }

    @Override
    protected void validate20XResponse(ClientResponse response, RestDescriptor descriptor) {
        String url = Config.Instance().securePushUrl + "api/register/v1/test/challenge";
        String body = response.getEntity(String.class);
        Action action = new Gson().fromJson(body, Action.class);
        assertEquals("My test listener.", action.description);
        assertTrue(action.contactEmail, action.contactEmail.equals("integrationtest@test.com"));
        assertEquals("My listener", action.name);
        assertEquals(url, action.url);
        assertEquals(url, action.img);
        assertEquals(1, action.filter.entityType.size());
        assertEquals("*", action.filter.entityType.get(0));

        assertEquals(1, action.filter.source.size());
        assertEquals("*", action.filter.source.get(0));

        assertEquals(1, action.filter.version.size());
        assertEquals("*", action.filter.version.get(0));
    }
}
