/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.util;

import com.google.gson.Gson;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.core.MediaType;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;

public abstract class TestUtils {
    protected static String opsToken = null;
    protected static String admToken = null;
    protected static String editorToken = null;
    protected static String noAccessToken = null;

    public static String getApiPath(String api) throws Exception {
        String baseUrl = Config.Instance().HostUrl;
        URL mergedURL = new URL(baseUrl + api);
        return mergedURL.toString();
    }

    public static String getOsduTenant() {
        return Config.Instance().OsduTenant;
    }

    public static String getCustomerTenant() {
        return Config.Instance().ClientTenant;
    }

    public abstract String getOpsAccessToken() throws Exception;

    public abstract String getAdmAccessToken() throws Exception;

    public abstract String getEditorAccessToken() throws Exception;

    public abstract String getNoDataAccessToken() throws Exception;

    public static ClientResponse send(String path, String httpMethod, String token, String requestBody, String query)
            throws Exception {

        Map<String, String> headers = getOsduTenantHeaders();

        return send(path, httpMethod, token, requestBody, query, headers);
    }

    public static ClientResponse sendHttp(String path, String httpMethod, String token, String requestBody, String query)
            throws Exception {

        Map<String, String> headers = getOsduTenantHeaders();

        return send(path, httpMethod, token, requestBody, query, headers, true);
    }

    public static Map<String, String> getOsduTenantHeaders() {
        Map<String, String> headers = new HashMap<>();
        headers.put("data-partition-id", getOsduTenant());
        return headers;
    }

    public static Map<String, String> getCustomerTenantHeaders() {
        Map<String, String> headers = new HashMap<>();
        headers.put("data-partition-id", getCustomerTenant());
        return headers;
    }

    public static ClientResponse send(String path, String httpMethod, String token, String requestBody, String query,
                                      Map<String, String> headers)
            throws Exception {
        return send(path, httpMethod, token, requestBody, query, headers, false);
    }


    public static ClientResponse send(String path, String httpMethod, String token, String requestBody, String query,
                                      Map<String, String> headers, boolean isHttp)
            throws Exception {

        Client client = getClient();
        client.setConnectTimeout(120000);
        client.setReadTimeout(120000);
        client.setFollowRedirects(false);
        String url = getApiPath(path + query);
        if (isHttp) {
            url = url.replaceFirst("https", "http");
        }
        System.out.println(url);
        System.out.println(httpMethod);
        System.out.println(token);
        System.out.println(requestBody);
        System.out.println(headers);

        WebResource webResource = client.resource(url);
        final WebResource.Builder builder = webResource.type(MediaType.APPLICATION_JSON)
                .header("Authorization", token);
        headers.forEach((k, v) -> builder.header(k, v));
        return builder.method(httpMethod, ClientResponse.class, requestBody);
    }

    @SuppressWarnings("unchecked")
    public static <T> T getResult(ClientResponse response, int exepectedStatus, Class<T> classOfT) {
        String json = response.getEntity(String.class);
        System.out.println(json);

        assertEquals(exepectedStatus, response.getStatus());
        if (exepectedStatus == 204) {
            return null;
        }

        assertEquals(MediaType.APPLICATION_JSON, response.getType().toString());
        if (classOfT == String.class) {
            return (T) json;
        }

        Gson gson = new Gson();
        return gson.fromJson(json, classOfT);
    }

    public static Client getClient() {
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return null;
            }

            @Override
            public void checkClientTrusted(X509Certificate[] certs, String authType) {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] certs, String authType) {
            }
        }};

        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
        }

        return Client.create();
    }
}